@@ -13,9 +13,6 @@ except ImportError:
     __version__ = 'unknown'
 import sys
 
-from pyroute2.cli.console import Console
-from pyroute2.cli.server import Server
-
 ##
 #
 # Logging setup
@@ -28,8 +25,8 @@ from pyroute2.cli.server import Server
 #  * https://github.com/svinota/pyroute2/issues/601
 #
 from pyroute2.config import log
-from pyroute2.conntrack import Conntrack, ConntrackEntry
-from pyroute2.devlink import DL
+from pyroute2.conntrack import AsyncConntrack, Conntrack, ConntrackEntry
+from pyroute2.devlink import DL, AsyncDL
 from pyroute2.ethtool.ethtool import Ethtool
 from pyroute2.ipdb import IPDB, CommitException, CreateException
 from pyroute2.iproute import (
@@ -40,31 +37,44 @@ from pyroute2.iproute import (
     NetNS,
     RawIPRoute,
 )
-from pyroute2.ipset import IPSet
-from pyroute2.ipvs import IPVS, IPVSDest, IPVSService
-from pyroute2.iwutil import IW
+from pyroute2.ipset import AsyncIPSet, IPSet
+from pyroute2.ipvs import IPVS, AsyncIPVS, IPVSDest, IPVSService
+from pyroute2.iwutil import IW, AsyncIW
 from pyroute2.ndb.main import NDB
 from pyroute2.netlink.connector.cn_proc import ProcEventSocket
-from pyroute2.netlink.devlink import DevlinkSocket
+from pyroute2.netlink.devlink import AsyncDevlinkSocket, DevlinkSocket
 from pyroute2.netlink.diag import DiagSocket, ss2
-from pyroute2.netlink.event.acpi_event import AcpiEventSocket
-from pyroute2.netlink.event.dquot import DQuotSocket
+from pyroute2.netlink.event import AsyncEventSocket, EventSocket
+from pyroute2.netlink.event.acpi_event import (
+    AcpiEventSocket,
+    AsyncAcpiEventSocket,
+)
+from pyroute2.netlink.event.dquot import AsyncDQuotSocket, DQuotSocket
+from pyroute2.netlink.event.thermal import (
+    AsyncThermalEventSocket,
+    ThermalEventSocket,
+)
 from pyroute2.netlink.exceptions import (
     ChaoticException,
     NetlinkDecodeError,
     NetlinkDumpInterrupted,
     NetlinkError,
 )
-from pyroute2.netlink.generic import GenericNetlinkSocket
-from pyroute2.netlink.generic.l2tp import L2tp
-from pyroute2.netlink.generic.mptcp import MPTCP
-from pyroute2.netlink.generic.wireguard import WireGuard
+from pyroute2.netlink.generic import (
+    AsyncGenericNetlinkSocket,
+    GenericNetlinkSocket,
+)
+from pyroute2.netlink.generic.ethtool import AsyncNlEthtool, NlEthtool
+from pyroute2.netlink.generic.ipvs import AsyncIPVSSocket, IPVSSocket
+from pyroute2.netlink.generic.l2tp import AsyncL2tp, L2tp
+from pyroute2.netlink.generic.mptcp import MPTCP, AsyncMPTCP
+from pyroute2.netlink.generic.wireguard import AsyncWireGuard, WireGuard
 from pyroute2.netlink.ipq import IPQSocket
-from pyroute2.netlink.nfnetlink.nfctsocket import NFCTSocket
+from pyroute2.netlink.nfnetlink.nfctsocket import AsyncNFCTSocket, NFCTSocket
 from pyroute2.netlink.nfnetlink.nftsocket import AsyncNFTSocket, NFTSocket
-from pyroute2.netlink.nl80211 import NL80211
+from pyroute2.netlink.nl80211 import NL80211, AsyncNL80211
 from pyroute2.netlink.rtnl.iprsocket import AsyncIPRSocket, IPRSocket
-from pyroute2.netlink.taskstats import TaskStats
+from pyroute2.netlink.taskstats import AsyncTaskStats, TaskStats
 from pyroute2.netlink.uevent import UeventSocket
 from pyroute2.nslink.nspopen import NSPopen
 from pyroute2.plan9.client import Plan9ClientSocket
@@ -84,54 +94,75 @@ if sys.platform.startswith('win'):  # noqa: E402
 
 modules = [
     AcpiEventSocket,
-    AsyncIPRoute,
+    AsyncAcpiEventSocket,
+    AsyncConntrack,
+    AsyncDL,
+    AsyncDQuotSocket,
+    AsyncDevlinkSocket,
+    AsyncEventSocket,
+    AsyncGenericNetlinkSocket,
     AsyncIPRSocket,
+    AsyncIPRoute,
+    AsyncIPSet,
+    AsyncIPVS,
+    AsyncIPVSSocket,
+    AsyncIW,
+    AsyncL2tp,
+    AsyncMPTCP,
+    AsyncNFCTSocket,
     AsyncNFTSocket,
+    AsyncNL80211,
+    AsyncNlEthtool,
+    AsyncTaskStats,
+    AsyncThermalEventSocket,
+    AsyncWireGuard,
     ChaoticException,
     ChaoticIPRoute,
     CommitException,
     Conntrack,
     ConntrackEntry,
-    Console,
     CreateException,
-    DevlinkSocket,
-    DiagSocket,
     DL,
     DQuotSocket,
+    DevlinkSocket,
+    DiagSocket,
     Ethtool,
+    EventSocket,
+    GenericNetlinkSocket,
     IPBatch,
     IPDB,
     IPQSocket,
-    IPRoute,
     IPRSocket,
+    IPRoute,
     IPSet,
     IPVS,
     IPVSDest,
     IPVSService,
+    IPVSSocket,
     IW,
-    GenericNetlinkSocket,
     L2tp,
     MPTCP,
     NDB,
-    NetlinkError,
-    NetlinkDecodeError,
-    NetlinkDumpInterrupted,
-    NetNS,
     NFCTSocket,
     NFTSocket,
     NL80211,
     NSPopen,
+    NetNS,
+    NetlinkDecodeError,
+    NetlinkDumpInterrupted,
+    NetlinkError,
+    NlEthtool,
     Plan9ClientSocket,
     Plan9ServerSocket,
     ProcEventSocket,
     RawIPRoute,
-    Server,
-    ss2,
     TaskStats,
+    ThermalEventSocket,
     UeventSocket,
-    WireGuard,
     WiSet,
+    WireGuard,
     log,
+    ss2,
 ]
 
 __all__ = []
