@@ -12,8 +12,13 @@ from pyroute2.ethtool.common import (
     LMBTypePort,
 )
 from pyroute2.ethtool.ioctl import WAKE_NAMES, IoctlEthtool
+from pyroute2.ethtool.module_info import ModuleInfoParser
 from pyroute2.netlink.exceptions import NetlinkError
-from pyroute2.netlink.generic.ethtool import NlEthtool, ethtool_rings_msg
+from pyroute2.netlink.generic.ethtool import (
+    NlEthtool,
+    ethtool_module_eeprom_msg,
+    ethtool_rings_msg,
+)
 
 INT32MINUS_UINT32 = c_uint32(-1).value
 INT16MINUS_UINT16 = c_uint16(-1).value
@@ -71,6 +76,31 @@ class EthtoolFeatures(namedtuple('EthtoolFeatures', ('features',))):
             ioctl_features[feature.name] = feature.enable
 
 
+class EthtoolChannels(
+    namedtuple(
+        'EthtoolChannels',
+        (
+            "max_rx",
+            "max_tx",
+            "max_other",
+            "max_combined",
+            "rx_count",
+            "tx_count",
+            "other_count",
+            "combined_count",
+        ),
+    )
+):
+    @classmethod
+    def from_ioctl(cls, channels):
+        return cls(**{k: getattr(channels, k) for k in cls._fields})
+
+    @staticmethod
+    def to_ioctl(ioctl_channels, eth_channel):
+        for key, val in eth_channel.items():
+            setattr(ioctl_channels, key, val)
+
+
 class EthtoolWakeOnLan(namedtuple('EthtoolWolMode', ('modes', 'sopass'))):
     @classmethod
     def from_netlink(cls, nl_wol):
@@ -357,6 +387,44 @@ class EthtoolRings(
         return cls(**ioctl_rings)
 
 
+class EthtoolModuleEeprom(
+    namedtuple(
+        "EthtoolModuleEeprom",
+        ("offset", "length", "page", "bank", "i2c_address", "data"),
+    )
+):
+    nl_pfx = "ETHTOOL_A_MODULE_EEPROM_{}"
+
+    def __new__(
+        cls, offset=0, length=0, page=0, bank=0, i2c_address=0, data=None
+    ):
+        return super(EthtoolModuleEeprom, cls).__new__(
+            cls, offset, length, page, bank, i2c_address, data
+        )
+
+    @classmethod
+    def from_netlink(cls, nl_eeprom):
+        nl_eeprom = nl_eeprom[0]
+        attrs = {
+            cls_attr: nl_eeprom.get_attr(cls.nl_pfx.format(cls_attr.upper()))
+            for cls_attr in cls._fields
+        }
+        if attrs["data"] is not None:
+            attrs["length"] = len(attrs["data"])
+        return cls(**attrs)
+
+    def to_netlink(self):
+        nl_module_eeprom_attrs = ethtool_module_eeprom_msg()
+        for cls_attr in self._fields:
+            cls_value = getattr(self, cls_attr)
+            if cls_value is None:
+                continue
+            nl_module_eeprom_attrs["attrs"].append(
+                (self.nl_pfx.format(cls_attr.upper()), cls_value)
+            )
+        return nl_module_eeprom_attrs
+
+
 class Ethtool:
     def __init__(self):
         self._with_ioctl = IoctlEthtool()
@@ -465,6 +533,16 @@ class Ethtool:
         EthtoolFeatures.to_ioctl(ioctl_features, features)
         self._with_ioctl.set_features(ioctl_features)
 
+    def get_channels(self, ifname):
+        self._with_ioctl.change_ifname(ifname)
+        return EthtoolChannels.from_ioctl(self._with_ioctl.get_channels())
+
+    def set_channels(self, ifname, channels):
+        self._with_ioctl.change_ifname(ifname)
+        ioctl_channels = self._with_ioctl.get_channels()
+        EthtoolChannels.to_ioctl(ioctl_channels, channels)
+        self._with_ioctl.set_channels(ioctl_channels)
+
     def get_coalesce(self, ifname):
         self._with_ioctl.change_ifname(ifname)
         return EthtoolCoalesce.from_ioctl(self._with_ioctl.get_coalesce())
@@ -475,6 +553,14 @@ class Ethtool:
         EthtoolCoalesce.to_ioctl(ioctl_coalesce, coalesce)
         self._with_ioctl.set_coalesce(ioctl_coalesce)
 
+    def get_module_eeprom(self, ifname, **kwargs):
+        eeprom_request = EthtoolModuleEeprom(**kwargs).to_netlink()
+        eeprom = self._with_nl.get_module_eeprom(eeprom_request, ifname)
+        return EthtoolModuleEeprom.from_netlink(eeprom)
+
+    def get_module_info(self, ifname):
+        return ModuleInfoParser(self, ifname).parse_module_info()
+
     def close(self):
         self._with_ioctl.close()
         self._with_nl.close()
