@@ -142,6 +142,16 @@ BRIDGE_FLAGS_SELF = 2
     'BRIDGE_FLAGS', globals()
 )
 
+##
+#
+# bridge mode
+#
+BRIDGE_MODE_UNSPEC = 0  # mode unspecified
+BRIDGE_MODE_HAIRPIN = 1  # hairpin mode on
+(BRIDGE_MODE_NAMES, BRIDGE_MODE_VALUES) = map_namespace(
+    'BRIDGE_MODE', globals()
+)
+
 ##
 #
 # XDP flags
@@ -458,6 +468,7 @@ class ifinfbase(object):
     #
     sql_constraints = {'index': 'NOT NULL'}
     sql_extra_fields = (('state', 'TEXT'),)
+    sql_ignore = {'IFLA_ALLMULTI'}
     lookup_fallbacks = {'index': 'ifname'}
 
     fields = (
@@ -531,6 +542,7 @@ class ifinfbase(object):
         ('IFLA_GRO_MAX_SIZE', 'uint32'),
         ('IFLA_TSO_MAX_SIZE', 'uint32'),
         ('IFLA_TSO_MAX_SEGS', 'uint32'),
+        ('IFLA_ALLMULTI', 'uint32'),
     )
 
     @staticmethod
